@@ -25,6 +25,7 @@
 #include "burn.h"
 #include <stdio.h>
 #include "media.h"
+#include "preferences.h"
 
 
 static const gchar* const widget_audiocd_tree = "treeview8";
@@ -37,25 +38,27 @@ static gdouble audiocdsize = 0.0;
 
 static DiskSize audiodisksizes[] = 
 {
-	{22, "22 min. CD"},
-	{74, "74 min. CD"},
-	{80, "80 min. CD"},
-	{90, "90 min. CD"}
+	{21.0, "21 min. CD"},
+	{63.0, "63 min. CD"},
+	{74.0, "74 min. CD"},
+    {80.0, "80 min. CD"},
+	{90.0, "90 min. CD"},
+    {99.0, "99 min. CD"}
 };
 
 
 enum
 {
+    TARGET_URI_LIST,
     TARGET_STRING,
-    TARGET_URL    
+    TARGET_COUNT
 };
 
 
 static GtkTargetEntry targetentries[] = 
 {
-    {"STRING", 0, TARGET_STRING},
-    {"text/plain", 0, TARGET_STRING},
-    {"text/uri-list", 0, TARGET_URL},
+    {"text/uri-list", 0, TARGET_URI_LIST},
+    {"text/plain", 0, TARGET_STRING}
 };
 
 
@@ -101,6 +104,20 @@ audiocd_get_audiocd_size()
 }
 
 
+static gchar*
+audiocd_format_progress_text(gdouble currentsecs)
+{
+    GB_LOG_FUNC
+    g_return_val_if_fail(currentsecs < (audiocdsize * 60), NULL);
+    
+    gint ss1 = ((gint)currentsecs)%60;
+    gint m1 = (((gint)currentsecs)-ss1)/60;
+    gint ss2 = ((gint)((audiocdsize * 60) - currentsecs))%60;
+    gint m2 = (((gint)((audiocdsize * 60) - currentsecs))-ss2)/60;        
+    return g_strdup_printf(_("%d mins %d secs used - %d mins %d secs remaining"), m1, ss1, m2, ss2);    
+}   
+
+
 static gboolean 
 audiocd_update_progress_bar(gboolean add, gdouble seconds)
 {
@@ -128,20 +145,18 @@ audiocd_update_progress_bar(gboolean add, gdouble seconds)
 	if(fraction < 0.0 || fraction == -0.0)
 	{
 		gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(progbar), 0.0);
-		gtk_progress_bar_set_text(GTK_PROGRESS_BAR(progbar), _("0 mins 0 secs"));
+        gchar* buf = audiocd_format_progress_text(0.0);
+		gtk_progress_bar_set_text(GTK_PROGRESS_BAR(progbar), buf);
+        g_free(buf);
 		gnomebaker_enable_widget(widget_audiocd_create, FALSE);
 	}	
 	/* If the file is too large then we don't allow the user to add it */
 	else if(fraction <= 1.0)
 	{
 		gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(progbar), fraction);
-
-		gint ss = ((gint)currentsecs)%60;
-		gint m = (((gint)currentsecs)-ss)/60;
-		
-		gchar* buf = g_strdup_printf(_("%d mins %d secs"), m, ss);
-		gtk_progress_bar_set_text(GTK_PROGRESS_BAR(progbar), buf);
-		g_free(buf);
+        gchar* buf = audiocd_format_progress_text(currentsecs);
+        gtk_progress_bar_set_text(GTK_PROGRESS_BAR(progbar), buf);
+        g_free(buf);
 		gnomebaker_enable_widget(widget_audiocd_create, TRUE);
 	}
 	else
@@ -273,7 +288,7 @@ audiocd_add_file(const gchar* filename, GtkTreeModel* model)
 	g_return_val_if_fail(filename != NULL, FALSE);
 	g_return_val_if_fail(model != NULL, FALSE);
     
-    GB_TRACE("audiocd_add_file - [%s]", filename);
+    GB_TRACE("audiocd_add_file - [%s]\n", filename);
 	/* ret is set to true as we want to ignore non audio files. */
 	gboolean ret = TRUE;
 	MediaInfo* info = media_info_new(filename);
@@ -364,7 +379,7 @@ audiocd_import_m3u_playlist(const gchar* m3ufile, GtkTreeModel* model)
     {
         gchar* file = *line;
         g_strstrip(file);
-        GB_TRACE("audiocd_import_m3u_playlist - [%s]", file);
+        GB_TRACE("audiocd_import_m3u_playlist - [%s]\n", file);
         if((strlen(file) > 0) && (file[0] != '#') && !g_ascii_isspace(file[0]))
             ret = audiocd_import_playlist_file(m3udir, file, model);
         ++line;
@@ -396,7 +411,7 @@ audiocd_import_pls_playlist(const gchar* plsfile, GtkTreeModel* model)
             if(file != NULL)
             {
                 ++file;            
-                GB_TRACE("audiocd_import_pls_playlist - [%s]", file);
+                GB_TRACE("audiocd_import_pls_playlist - [%s]\n", file);
                 ret = audiocd_import_playlist_file(plsdir, file, model);
             }
             else
@@ -481,6 +496,11 @@ audiocd_on_audiocd_size_changed(GtkOptionMenu *optionmenu, gpointer user_data)
 		
 	fraction = (fraction * previoussize)/audiocdsize;
 	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(progbar), fraction);	
+    gchar* buf = audiocd_format_progress_text(fraction * audiocdsize* 60);
+    gtk_progress_bar_set_text(GTK_PROGRESS_BAR(progbar), buf);
+    g_free(buf);
+    
+    preferences_set_int(GB_AUDIO_DISK_SIZE, gtk_option_menu_get_history(optionmenu));
 }
 
 
@@ -635,8 +655,11 @@ audiocd_clear()
     GtkTreeModel* filemodel = gtk_tree_view_get_model(GTK_TREE_VIEW(audiotree));    
     GtkWidget* progbar = glade_xml_get_widget(gnomebaker_getxml(), widget_audiocd_progressbar);
     
+    gchar* buf = audiocd_format_progress_text(0.0);
+    gtk_progress_bar_set_text(GTK_PROGRESS_BAR(progbar), buf);
+    g_free(buf);
     gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(progbar), 0.0);
-    gtk_progress_bar_set_text(GTK_PROGRESS_BAR(progbar), _("0 mins 0 secs"));
+
     gnomebaker_enable_widget(widget_audiocd_create, FALSE);
 
     /* Now get each row and free the MediaInfo* we store in each */    
@@ -672,7 +695,7 @@ gboolean
 audiocd_import_playlist(const gchar* playlist)
 {
     GB_LOG_FUNC
-    g_return_if_fail(playlist != NULL);
+    g_return_val_if_fail(playlist != NULL, FALSE);
     
     gboolean ret = FALSE;
     gchar* mime = gbcommon_get_mime_type(playlist);
@@ -686,6 +709,118 @@ audiocd_import_playlist(const gchar* playlist)
 }
 
 
+static gboolean
+audiocd_export_m3u(const gchar* playlist)
+{
+    GB_LOG_FUNC
+    g_return_val_if_fail(playlist != NULL, FALSE);
+
+    gboolean ret = FALSE;
+    FILE *file = NULL;   
+    if ((file = fopen(playlist, "w")) == 0)
+    {
+        g_critical("Failed to write playlist [%s]", playlist);
+        gnomebaker_show_msg_dlg(NULL, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK,
+            GTK_BUTTONS_NONE, _("Failed to write playlist file"));
+    }
+    else
+    {
+        GtkWidget* tree = glade_xml_get_widget(gnomebaker_getxml(), widget_audiocd_tree);
+        GtkTreeModel* model = gtk_tree_view_get_model(GTK_TREE_VIEW(tree));
+        GtkTreeIter iter;
+        gboolean valid = gtk_tree_model_get_iter_first (model, &iter);
+        while (valid)
+        {
+            gchar *playlist_item = NULL;
+            gtk_tree_model_get (model, &iter,  AUDIOCD_COL_FILE, &playlist_item, -1);
+            fprintf(file, "%s\n", playlist_item);
+            g_free (playlist_item);
+            valid = gtk_tree_model_iter_next (model, &iter);
+        }
+        
+        fclose(file);
+        ret = TRUE;
+    }
+    
+    return ret;
+}    
+
+
+static gboolean
+audiocd_export_pls(const gchar* playlist)
+{
+    GB_LOG_FUNC
+    g_return_val_if_fail(playlist != NULL, FALSE);
+
+    gboolean ret = FALSE;
+    FILE *file;   
+    if ((file = fopen(playlist, "w")) == 0)
+    {
+        g_critical("Failed to write playlist [%s]", playlist);
+        gnomebaker_show_msg_dlg(NULL, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK, 
+            GTK_BUTTONS_NONE, _("Failed to write playlist file"));
+    }
+    else
+    {
+        fprintf(file, "[playlist]\n");
+        GtkWidget* tree = glade_xml_get_widget(gnomebaker_getxml(), widget_audiocd_tree);
+        GtkTreeModel* model = gtk_tree_view_get_model(GTK_TREE_VIEW(tree));
+        GtkTreeIter iter;
+        gboolean valid = gtk_tree_model_get_iter_first (model, &iter);
+        gint track = 1;
+        while (valid)
+        {
+            gchar *playlist_item = NULL, *title = NULL;
+            MediaInfo* info = NULL;
+            gtk_tree_model_get (model, &iter, AUDIOCD_COL_FILE, &playlist_item, -1);
+            gtk_tree_model_get (model, &iter, AUDIOCD_COL_TITLE, &title, -1);
+            gtk_tree_model_get (model, &iter, AUDIOCD_COL_INFO, &info, -1);
+            fprintf(file, "File%d=%s\n", track, playlist_item);
+            fprintf(file, "Title%d=%s\n", track, title);
+            fprintf(file, "Length%d=%ld\n", track, info->duration);
+            g_free (playlist_item);
+            g_free (title);
+            /*g_free (info); we don't own the media info pointer */
+            
+            valid = gtk_tree_model_iter_next (model, &iter);
+            track++;
+        }
+
+        fprintf(file, "NumberOfEntries=%d\n", track-1);
+        fprintf(file, "Version=2");
+        fclose(file);
+        ret = TRUE;
+    }
+    
+    return ret;
+}    
+
+
+gboolean 
+audiocd_export_playlist(const gchar* playlist)
+{
+    GB_LOG_FUNC
+    g_return_val_if_fail(playlist != NULL, FALSE);
+
+    gboolean ret = FALSE;
+    gint result = GTK_RESPONSE_YES;
+    if(g_file_test(playlist, G_FILE_TEST_EXISTS))
+    {
+        result = gnomebaker_show_msg_dlg(NULL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO,
+                    GTK_BUTTONS_NONE, _("Playlist already exists.\n Do you want to replace it?"));
+    }
+
+    if(result == GTK_RESPONSE_YES)
+    {
+        if (strlen(playlist) < 4 || gbcommon_str_has_suffix(playlist, ".m3u"))
+            ret = audiocd_export_m3u(playlist);
+        else if (strlen(playlist) < 4 || gbcommon_str_has_suffix(playlist, ".pls"))
+            ret = audiocd_export_pls(playlist);
+    }   
+    return ret;
+}
+
+
 void
 audiocd_new()
 {
@@ -771,7 +906,7 @@ audiocd_new()
 
     /* Enable the file list as a drag destination */    
     gtk_drag_dest_set(GTK_WIDGET(filelist), GTK_DEST_DEFAULT_ALL,
-        targetentries, 3, GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK);
+        targetentries, TARGET_COUNT, GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK);
 
     /* Connect the function to handle the drag data */
     g_signal_connect(filelist, "drag_data_received",
@@ -786,6 +921,6 @@ audiocd_new()
         
     GtkWidget* optmenu = glade_xml_get_widget(gnomebaker_getxml(), widget_audiocd_size);
     gbcommon_populate_disk_size_option_menu(GTK_OPTION_MENU(optmenu), audiodisksizes, 
-        (sizeof(audiodisksizes)/sizeof(DiskSize)), 2);
+        (sizeof(audiodisksizes)/sizeof(DiskSize)), preferences_get_int(GB_AUDIO_DISK_SIZE));
 }
 
