@@ -21,6 +21,7 @@
 
 #include "datacd.h"
 #include "gnomebaker.h"
+#include <sys/types.h>
 #include <sys/stat.h>
 #include "filebrowser.h"
 #include "gbcommon.h"
@@ -55,33 +56,35 @@ enum
 };
 
 
-DiskSize datadisksizes[DISK_SIZE_COUNT] = 
+static DiskSize datadisksizes[DISK_SIZE_COUNT] = 
 {
-	{200 * 1024 * 1024, "200MB CD"},
-	{650 * 1024 * 1024, "650MB CD"},
-	{700 * 1024 * 1024, "700MB CD"},
-	{800 * 1024 * 1024, "800MB CD"},
-	{4.7 * 1000 * 1000 * 1000, "4.7GB DVD"}, /* DVDs are salesman's MegaByte ie 1000 not 1024 */
-	{8.5 * 1000 * 1000 * 1000, "8.5GB DVD"}
+    /* http://www.cdrfaq.org/faq07.html#S7-6 
+        http://www.osta.org/technology/dvdqa/dvdqa6.htm */
+	{94500.0 * 2048, "200MB CD"},
+	{333000.0 * 2048, "650MB CD"},
+	{360000.0 * 2048, "700MB CD"},
+	{405000.0 * 2048, "800MB CD"},
+	{2294922.0 * 2048, "4.7GB DVD"}, 
+	{8.5 * 1000 * 1000 * 1000, "8.5GB DVD"} /* DVDs are salesman's MegaByte ie 1000 not 1024 */
 };
 
 
 enum
 {
+    TARGET_URI_LIST,
     TARGET_STRING,
-    TARGET_URL
+    TARGET_COUNT
 };
 
 
 static GtkTargetEntry targetentries[] = 
 {
-    {"STRING", 0, TARGET_STRING},
-    {"text/plain", 0, TARGET_STRING},
-    {"text/uri-list", 0, TARGET_URL},
+    {"text/uri-list", 0, TARGET_URI_LIST},
+    {"text/plain", 0, TARGET_STRING}
 };
 
 
-void 
+static void 
 datacd_set_multisession(const gchar* msinfo)
 {
 	GB_LOG_FUNC
@@ -95,7 +98,7 @@ datacd_set_multisession(const gchar* msinfo)
 }
 
 
-void
+static void
 datacd_on_show_humansize_changed(GConfClient *client,
                                    guint cnxn_id,
 								   GConfEntry *entry,
@@ -122,7 +125,7 @@ datacd_on_show_humansize_changed(GConfClient *client,
 }
 
 
-gdouble 
+static gdouble 
 datacd_get_datadisk_size()
 {
 	GB_LOG_FUNC
@@ -133,7 +136,22 @@ datacd_get_datadisk_size()
 }
 
 
-gboolean 
+static gchar*
+datacd_format_progress_text(gdouble currentsize)
+{
+    GB_LOG_FUNC
+    g_return_val_if_fail(currentsize < datadisksize, NULL);
+    
+    gchar* current = gbcommon_humanreadable_filesize((guint64)currentsize);
+    gchar* remaining = gbcommon_humanreadable_filesize((guint64)(datadisksize - currentsize));
+    gchar* buf = g_strdup_printf(_("%s used - %s remaining"), current, remaining);
+    g_free(current);
+    g_free(remaining);
+    return buf;
+}   
+
+
+static gboolean 
 datacd_update_progress_bar(gboolean add, guint64 filesize)
 {
 	GB_LOG_FUNC
@@ -156,11 +174,13 @@ datacd_update_progress_bar(gboolean add, guint64 filesize)
 		currentsize -= (gdouble)filesize;
 	
 	fraction = currentsize / disksize;
-	
+    
 	if(fraction < 0.0 || fraction == -0.0)
 	{
 		gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(progbar), 0.0);
-		gtk_progress_bar_set_text(GTK_PROGRESS_BAR(progbar), "0%");
+		gchar* buf = datacd_format_progress_text(0.0);
+        gtk_progress_bar_set_text(GTK_PROGRESS_BAR(progbar), buf);
+        g_free(buf);
 		
 		/* disable the create button as there's nothing on the disk */
 		gnomebaker_enable_widget(widget_datacd_create, FALSE);
@@ -171,11 +191,10 @@ datacd_update_progress_bar(gboolean add, guint64 filesize)
 	/* If the file is too large then we don't allow the user to add it */
 	else if(fraction <= 1.0)
 	{
-		gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(progbar), fraction);
-		
-		gchar* buf = g_strdup_printf("%d%%", (gint)(fraction * 100));
-		gtk_progress_bar_set_text(GTK_PROGRESS_BAR(progbar), buf);
-		g_free(buf);
+		gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(progbar), fraction);		
+        gchar* buf = datacd_format_progress_text(currentsize);
+        gtk_progress_bar_set_text(GTK_PROGRESS_BAR(progbar), buf);
+        g_free(buf);
 		gnomebaker_enable_widget(widget_datacd_create, TRUE);
 	}
 	else
@@ -184,12 +203,11 @@ datacd_update_progress_bar(gboolean add, guint64 filesize)
 			_("File or directory is too large to fit in the remaining space on the CD"));
 		ok = FALSE;
 	}
-	
 	return ok;
 }
 
 
-gboolean  
+static gboolean  
 datacd_add_to_compilation(const gchar* file, GtkListStore* liststore, gboolean existingsession)
 {
 	GB_LOG_FUNC
@@ -224,10 +242,11 @@ datacd_add_to_compilation(const gchar* file, GtkListStore* liststore, gboolean e
 				icon = gbcommon_get_icon_for_mime(mime, 16);
 				g_free(mime);
 			}
+			gchar* humanreadable = gbcommon_humanreadable_filesize(size);
 			gtk_list_store_set(liststore, &iter, DATACD_COL_ICON, icon,	DATACD_COL_FILE, basename, 
-				DATACD_COL_SIZE, size, DATACD_COL_HUMANSIZE, gbcommon_humanreadable_filesize(size),
+				DATACD_COL_SIZE, size, DATACD_COL_HUMANSIZE, humanreadable,
 				DATACD_COL_PATH, filename, DATACD_COL_SESSION, existingsession, -1);
-			
+			g_free(humanreadable);
 			g_object_unref(icon);
 			g_free(basename);
 		}
@@ -263,7 +282,7 @@ datacd_add_selection(GtkSelectionData* selection)
 	
 	gnomebaker_show_busy_cursor(TRUE);	    	
 
-	GB_TRACE("received sel %s", selection->data);	
+	GB_TRACE("datacd_add_selection - received sel [%s]\n", selection->data);	
 	const gchar* file = strtok((gchar*)selection->data,"\n");
 	while(file != NULL)
 	{
@@ -276,7 +295,7 @@ datacd_add_selection(GtkSelectionData* selection)
 }
 
 
-void
+static void
 datacd_on_drag_data_received(
     GtkWidget * widget,
     GdkDragContext * context,
@@ -292,7 +311,7 @@ datacd_on_drag_data_received(
 }
 
 
-void
+static void
 datacd_foreach_fileselection(GtkTreeModel *filemodel,
 								  GtkTreePath *path,
 								  GtkTreeIter *iter,
@@ -362,7 +381,7 @@ datacd_remove()
 }
 
 
-void
+static void
 datacd_on_remove_clicked(GtkWidget *menuitem, gpointer userdata)
 {
 	GB_LOG_FUNC	
@@ -384,7 +403,9 @@ datacd_clear()
 	GtkWidget* progbar = glade_xml_get_widget(xml, widget_datacd_progressbar);
 	
 	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(progbar), 0.0);
-	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(progbar), "0%");
+	gchar* buf = datacd_format_progress_text(0.0);
+    gtk_progress_bar_set_text(GTK_PROGRESS_BAR(progbar), buf);
+    g_free(buf);
 	gnomebaker_enable_widget(widget_datacd_create, FALSE);
 	
 	/* clear any multisession flags */	
@@ -394,7 +415,7 @@ datacd_clear()
 }
 
 
-void
+static void
 datacd_on_clear_clicked(GtkWidget *menuitem, gpointer userdata)
 {	
 	GB_LOG_FUNC
@@ -402,7 +423,7 @@ datacd_on_clear_clicked(GtkWidget *menuitem, gpointer userdata)
 }
 
 
-void
+static void
 datacd_contents_cell_edited(GtkCellRendererText *cell,
 							gchar* path_string,
 							gchar* new_text,
@@ -428,7 +449,7 @@ datacd_contents_cell_edited(GtkCellRendererText *cell,
 }
 
 
-void 
+static void 
 datacd_on_edit(gpointer widget, gpointer user_data)
 {
 	GB_LOG_FUNC
@@ -437,7 +458,7 @@ datacd_on_edit(gpointer widget, gpointer user_data)
 }
 
 
-gboolean
+static gboolean
 datacd_on_button_pressed(GtkWidget *widget, GdkEventButton *event, gpointer user_data)
 {
 	GB_LOG_FUNC
@@ -556,7 +577,7 @@ datacd_new()
 
 	/* Enable the file list as a drag destination */	
     gtk_drag_dest_set(GTK_WIDGET(filelist), GTK_DEST_DEFAULT_ALL,
-		targetentries, 3, GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK);
+		targetentries, TARGET_COUNT, GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK);
 
 	/* Connect the function to handle the drag data */
     g_signal_connect(filelist, "drag_data_received",
@@ -570,38 +591,37 @@ datacd_new()
 	
 	GtkWidget* optmenu = glade_xml_get_widget(gnomebaker_getxml(), widget_datacd_size);
 	gbcommon_populate_disk_size_option_menu(GTK_OPTION_MENU(optmenu), datadisksizes, 
-		DISK_SIZE_COUNT, 2);
-	
-	/* get the currently selected cd size */
-	datacd_get_datadisk_size();	
+		DISK_SIZE_COUNT, preferences_get_int(GB_DATA_DISK_SIZE));
 }
 
 
-gboolean 
+static gboolean 
 datacd_get_msinfo(gchar** msinfo)
 {
 	GB_LOG_FUNC
 	gboolean ok = FALSE;
+    devices_unmount_device(GB_WRITER);
 	gchar* writer = devices_get_device_config(GB_WRITER, GB_DEVICE_ID_LABEL);
-	gchar* msinfocmd = g_strdup_printf("cdrecord -msinfo dev=%s", writer);
-	GString* output = exec_run_cmd(msinfocmd);
+	gchar* msinfocmd = g_strdup_printf("cdrecord -msinfo dev=%s", writer);    
+	gchar* output = NULL;
+    exec_run_cmd(msinfocmd, &output);
 	
 	gint start = 0, end = 0;
-	if((output == NULL ) || (sscanf(output->str, "%d,%d\n", &start, &end) != 2))
+	if((output == NULL ) || (sscanf(output, "%d,%d\n", &start, &end) != 2))
 	{
 		gchar* message = g_strdup_printf(_("Error getting session information.\n\n%s"), 
-			output != NULL ? output->str : _("unknown error"));
+			output != NULL ? output : _("unknown error"));
 		gnomebaker_show_msg_dlg(NULL, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, GTK_BUTTONS_NONE, message);
 		g_free(message);
 	}
 	else
 	{
 		*msinfo = g_strdup_printf("%d,%d", start, end);
-		GB_TRACE("datacd next session is [%s]", *msinfo);		
+		GB_TRACE("datacd_get_msinfo - next session is [%s]\n", *msinfo);		
 		ok = TRUE;
 	}
 	
-	g_string_free(output, TRUE);
+	g_free(output);
 	g_free(writer);
 	g_free(msinfocmd);
 	return ok;	
@@ -619,7 +639,7 @@ datacd_import_session()
 	
 	if(response == GTK_RESPONSE_CANCEL)
 		return;
-    if(devices_prompt_for_disk(NULL, GB_WRITER) == GTK_RESPONSE_CANCEL)
+    if(devices_prompt_for_disk(gnomebaker_get_window(), GB_WRITER) == GTK_RESPONSE_CANCEL)
         return;
 	
 	gnomebaker_show_busy_cursor(TRUE);
@@ -669,8 +689,7 @@ datacd_import_session()
 			g_free(message);			
 		}				
 		
-		if(!devices_mount_device(GB_WRITER, NULL))
-			g_critical("Error unmounting writer device");	
+		devices_unmount_device(GB_WRITER);
 	}
 	
 	g_free(mountpoint);	
@@ -680,7 +699,7 @@ datacd_import_session()
 }
 
 
-void 
+void /* libglade callback */
 datacd_on_datadisk_size_changed(GtkOptionMenu *optionmenu, gpointer user_data)
 {
 	GB_LOG_FUNC
@@ -694,28 +713,35 @@ datacd_on_datadisk_size_changed(GtkOptionMenu *optionmenu, gpointer user_data)
 		
 	fraction = (fraction * previoussize)/datadisksize;
 	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(progbar), fraction);
-	
-	gchar* buf = g_strdup_printf("%d%%", (gint)(fraction * 100));
+	gchar* buf = datacd_format_progress_text(fraction * datadisksize);
 	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(progbar), buf);
 	g_free(buf);
+    
+    preferences_set_int(GB_DATA_DISK_SIZE, gtk_option_menu_get_history(optionmenu));
 }
 
 
-void
+void /* libglade callback */
 datacd_on_create_datadisk(gpointer widget, gpointer user_data)
 {
 	GB_LOG_FUNC
 	
-	/* Here we should get a glist of data files to burn to the cd.
-	 * or something like that */	
 	GtkWidget *datatree = glade_xml_get_widget(gnomebaker_getxml(), widget_datacd_tree);
 	g_return_if_fail(datatree != NULL);
-	
 	GtkTreeModel* datamodel = gtk_tree_view_get_model(GTK_TREE_VIEW(datatree));
 	g_return_if_fail(datamodel != NULL);
+    gchar* msinfo = (gchar*)g_object_get_data(G_OBJECT(datamodel), DATACD_EXISTING_SESSION);
 
 	if(datadisksize >= datadisksizes[DVD_4GB].size)
-		burn_create_data_dvd(datamodel);
-	else
-		burn_create_data_cd(datamodel);	
+    {
+        if(msinfo != NULL)
+            burn_append_data_dvd(datamodel);    
+        else
+		    burn_create_data_dvd(datamodel);
+    }
+	else if(msinfo != NULL)
+        burn_append_data_cd(datamodel);
+    else
+	    burn_create_data_cd(datamodel);	
+
 }
