@@ -32,7 +32,7 @@
 #include <fcntl.h>
 #include <stdlib.h>
 
-#ifdef __FreeBSD__
+#if defined(__FreeBSD__) || defined(__FreeBSD_kernel__)
 #include <sys/cdio.h>
 #define CDROM_DRIVE_STATUS     0x5326
 #define CDS_NO_DISC        1
@@ -88,7 +88,7 @@ devices_write_device_to_gconf(const gint devicenumber, const gchar* devicename,
 	g_free(devicenodekey);
 	g_free(devicemountkey);
 	g_free(devicecapabilitieskey);
-	GB_TRACE("devices_write_device_to_gconf - Added [%s] [%s] [%s] [%s]", 
+	GB_TRACE("devices_write_device_to_gconf - Added [%s] [%s] [%s] [%s]\n", 
 		devicename, deviceid, devicenode, mountpoint);
 }
 
@@ -114,7 +114,7 @@ devices_add_device(const gchar* devicename, const gchar* deviceid,
 			gchar node[64], mount[64];
 			if(sscanf(*line, "%s\t%s", node, mount) == 2)
 			{
-				GB_TRACE("node [%s] mount [%s]", node, mount);
+				GB_TRACE("devices_add_device - node [%s] mount [%s]\n", node, mount);
 				if(g_ascii_strcasecmp(node, devicenode) == 0)
 				{
 					mountpoint = g_strdup(mount);
@@ -127,7 +127,7 @@ devices_add_device(const gchar* devicename, const gchar* deviceid,
 					realpath(node, linktarget);					
 					if(g_ascii_strcasecmp(linktarget, devicenode) == 0)
 					{					
-						GB_TRACE("node [%s] is link to [%s]", node, linktarget);
+						GB_TRACE("devices_add_device - node [%s] is link to [%s]\n", node, linktarget);
 						mountpoint = g_strdup(mount);
 					}
 					g_free(linktarget);
@@ -370,15 +370,15 @@ devices_probe_bus(const gchar* bus)
 		strcat(command, bus);
 	}
 	
-	GString* buffer = exec_run_cmd(command);
+	gchar* buffer = NULL;
+    exec_run_cmd(command, &buffer);
 	if(buffer == NULL)
 		g_critical("devices_probe_bus - Failed to scan the scsi bus");
-	else if(!devices_parse_cdrecord_output(buffer->str, bus))	
+	else if(!devices_parse_cdrecord_output(buffer, bus))
 		g_critical("devices_probe_bus - failed to parse cdrecord output");
 	else
-		ok = TRUE;
-	
-	g_string_free(buffer, TRUE);
+		ok = TRUE;	
+	g_free(buffer);
 
 	return ok;
 }
@@ -392,7 +392,7 @@ devices_get_ide_device(const gchar* devicenode, const gchar* devicenodepath,
 	g_return_if_fail(devicenode != NULL);	
 	g_return_if_fail(modelname != NULL);
 	g_return_if_fail(deviceid != NULL);
-	GB_TRACE("devices_get_ide_device - probing [%s]", devicenode);
+	GB_TRACE("devices_get_ide_device - probing [%s]\n", devicenode);
 	gchar* contents = NULL;
 	gchar* file = g_strdup_printf("/proc/ide/%s/model", devicenode);
 	if(g_file_get_contents(file, &contents, NULL, NULL))
@@ -418,7 +418,7 @@ devices_get_scsi_device(const gchar* devicenode, const gchar* devicenodepath,
 	g_return_if_fail(devicenode != NULL);
 	g_return_if_fail(modelname != NULL);
 	g_return_if_fail(deviceid != NULL);
-	GB_TRACE("devices_add_scsi_device - probing [%s]", devicenode);
+	GB_TRACE("devices_add_scsi_device - probing [%s]\n", devicenode);
 	
 	gchar **device_strs = NULL, **devices = NULL;	
 	if((devices = gbcommon_get_file_as_list("/proc/scsi/sg/devices")) == NULL)
@@ -481,7 +481,7 @@ devices_get_scsi_device(const gchar* devicenode, const gchar* devicenodepath,
 void 
 devices_for_each(gpointer key, gpointer value, gpointer user_data)
 {	
-	GB_TRACE("---- key [%s], value [%s]", (gchar*)key, (gchar*)value);
+	GB_TRACE("---- key [%s], value [%s]\n", (gchar*)key, (gchar*)value);
 	g_free(key);
 	g_free(value);
 }
@@ -494,7 +494,7 @@ devices_get_cdrominfo(gchar** proccdrominfo, gint deviceindex)
 	g_return_val_if_fail(proccdrominfo != NULL, NULL);
 	g_return_val_if_fail(deviceindex >= 1, NULL);
 	
-	GB_TRACE("looking for device [%d]", deviceindex);
+	GB_TRACE("devices_get_cdrominfo - looking for device [%d]\n", deviceindex);
 	GHashTable* ret = NULL;
 	gchar** info = proccdrominfo;
 	while(*info != NULL)
@@ -529,8 +529,8 @@ devices_get_cdrominfo(gchar** proccdrominfo, gint deviceindex)
 				 looking for */
 				if(columnindex <= deviceindex)
 				{
-					GB_TRACE("Requested device index [%d] is out of bounds. "
-						"All devices have been read.", deviceindex);
+					GB_TRACE("devices_get_cdrominfo - Requested device index [%d] is out of bounds. "
+						"All devices have been read.\n", deviceindex);
 					g_hash_table_destroy(ret);
 					ret = NULL;
 					break;
@@ -615,51 +615,56 @@ devices_probe_busses()
 }
 
 
+void 
+devices_unmount_device(const gchar* devicekey)
+{
+    GB_LOG_FUNC
+    g_return_if_fail(devicekey != NULL);
+    
+    gchar* node = devices_get_device_config(devicekey, GB_DEVICE_NODE_LABEL);
+    gchar* mountcmd = g_strdup_printf("umount %s", node); 
+    gchar* output = NULL;
+    exec_run_cmd(mountcmd, &output);
+    g_free(output);
+    g_free(mountcmd);
+    g_free(node);
+}
+
+
 gboolean 
 devices_mount_device(const gchar* devicekey, gchar** mountpoint)
 {
 	GB_LOG_FUNC
 	g_return_val_if_fail(devicekey != NULL, FALSE);
+    g_return_val_if_fail(mountpoint != NULL, FALSE);
 	gboolean ok = FALSE;
 	
 	gchar* mount = devices_get_device_config(devicekey, GB_DEVICE_MOUNT_LABEL);		
 	if((mount == NULL) || (strlen(mount) == 0))
 	{
-        if(mountpoint != NULL)
-        {
-            gnomebaker_show_msg_dlg(NULL, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, GTK_BUTTONS_NONE,
-                _("The mount point (e.g. /mnt/cdrom) for the writing device could not be obtained. "
-                "Please check that the writing device has an entry in /etc/fstab and then go "
-                "to preferences and rescan for devices."));
-        }
+        gnomebaker_show_msg_dlg(NULL, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, GTK_BUTTONS_NONE,
+            _("The mount point (e.g. /mnt/cdrom) for the writing device could not be obtained. "
+            "Please check that the writing device has an entry in /etc/fstab and then go "
+            "to preferences and rescan for devices."));
 	}
 	else
 	{
-		gchar* mountcmd = NULL;
-		if(mountpoint != NULL)
-			mountcmd = g_strdup_printf("mount %s", mount);	
-		else
-			mountcmd = g_strdup_printf("umount %s", mount);	
-		
-		GString* output = exec_run_cmd(mountcmd);		
-		if((output == NULL) || ((strlen(output->str) > 0) && 
-				(strstr(output->str, "already mounted") == NULL) && 
-					(strstr(output->str, "is not mounted") == NULL)))
+		gchar* mountcmd = g_strdup_printf("mount %s", mount);	
+		gchar* output = NULL;
+		if(exec_run_cmd(mountcmd, &output) != 0)
 		{
-			gchar* message = g_strdup_printf(_("Error %s %s.\n\n%s"), 
-				mount ? _("mounting") : _("unmounting"), mount, output != NULL ? output->str : _("unknown error"));
+			gchar* message = g_strdup_printf(_("Error mounting %s.\n\n%s"), 
+	            mount, output != NULL ? output : _("unknown error"));
 			gnomebaker_show_msg_dlg(NULL, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, GTK_BUTTONS_NONE, message);
 			g_free(message);
 		}
 		else
 		{
-			ok = TRUE;
-			if(mountpoint != NULL)
-				*mountpoint = g_strdup(mount);
-		}
-		
-		g_string_free(output, TRUE);
-		g_free(mountcmd);		
+			ok = TRUE;			
+    		*mountpoint = g_strdup(mount);
+		}		
+		g_free(output);
+		g_free(mountcmd);
 	}
 	g_free(mount);
 	
@@ -676,7 +681,7 @@ devices_eject_disk(const gchar* devicekey)
 	/* from http://leapster.org/linux/cdrom/ */
 	gboolean ret = FALSE;
 	gchar *device = devices_get_device_config(devicekey,GB_DEVICE_NODE_LABEL);
-	GB_TRACE("Ejecting media in %s",device);
+	GB_TRACE("devices_eject_disk - Ejecting media in [%s]\n",device);
     int cdrom = open(device,O_RDONLY | O_NONBLOCK);
     g_free(device);
 	if(cdrom < 0)
@@ -686,7 +691,7 @@ devices_eject_disk(const gchar* devicekey)
     else
     {			
         /* Use ioctl to send the CDROMEJECT (CDIOCEJECT on FreeBSD) command to the device */
-#ifdef __FreeBSD__
+#if defined(__FreeBSD__) || defined(__FreeBSD_kernel__)
         if(ioctl(cdrom, CDIOCEJECT, 0) < 0)
             ret = TRUE;
         else 
@@ -779,7 +784,8 @@ devices_prompt_for_disk(GtkWindow* parent, const gchar* devicekey)
     gchar* devicename = devices_get_device_config(devicekey, GB_DEVICE_NAME_LABEL);
     gchar* message = g_strdup_printf(_("Please insert a disk into the %s"), devicename);    
     gint ret = GTK_RESPONSE_OK;
-    while(!devices_is_disk_inserted(devicekey) && (ret == GTK_RESPONSE_OK))
+    /*while(!devices_is_disk_inserted(devicekey) && (ret == GTK_RESPONSE_OK))*/
+    if(!devices_is_disk_inserted(devicekey))
     {
         devices_eject_disk(devicekey);
         ret = gnomebaker_show_msg_dlg(parent, GTK_MESSAGE_INFO, 
@@ -791,3 +797,17 @@ devices_prompt_for_disk(GtkWindow* parent, const gchar* devicekey)
 }
 
 
+gboolean 
+devices_reader_is_also_writer()
+{
+    GB_LOG_FUNC    
+    gchar* reader = devices_get_device_config(GB_READER, GB_DEVICE_NODE_LABEL);
+    gchar* writer = devices_get_device_config(GB_WRITER, GB_DEVICE_NODE_LABEL);
+    gboolean ret = (g_ascii_strcasecmp(reader, writer) == 0);
+    g_free(reader);
+    g_free(writer);
+    return ret;
+}
+
+
+
