@@ -33,9 +33,9 @@ static const gchar* const widget_browser_filelist = "treeview5";
 
 
 /* callback id, so we can block it! */
-gulong sel_changed_id;
+static gulong sel_changed_id;
 
-GtkTreePath* dirtreeselected = NULL;
+static GtkTreePath* dirtreeselected = NULL;
 
 enum /* Directory Tree */
 {
@@ -57,15 +57,16 @@ enum /* FileList */
 
 enum
 {
+    TARGET_URI_LIST,
     TARGET_STRING,
-    TARGET_URL
+    TARGET_COUNT
 };
 
+
 static GtkTargetEntry targetentries[] = 
 {
-    {"STRING", 0, TARGET_STRING},
-    {"text/plain", 0, TARGET_STRING},
-    {"text/uri-list", 0, TARGET_URL},
+    {"text/uri-list", 0, TARGET_URI_LIST},
+    {"text/plain", 0, TARGET_STRING}
 };
 
 
@@ -315,7 +316,7 @@ filebrowser_populate(GtkTreeModel* treemodel,
 						GdkPixbuf* icon = gbcommon_get_icon_for_name("gnome-fs-directory", 16);
 						gtk_list_store_set(GTK_LIST_STORE(filemodel), &iterRight, 
 							FL_COL_ICON, icon, FL_COL_NAME, name,
-							FL_COL_SIZE, (guint64)0, FL_COL_HUMANSIZE, "4 B", 
+							FL_COL_SIZE, (guint64)4096, FL_COL_HUMANSIZE, "4 KB",
 							FL_COL_TYPE, DIRECTORY, FL_COL_MIME, "x-directory/normal", -1);
 						g_object_unref(icon);
 					}
@@ -498,7 +499,7 @@ filebrowser_on_drag_data_get(GtkWidget* widget, GdkDragContext* context,
 		g_string_assign(file, uri);
 	}
 	
-	GB_TRACE("selection data is %s\n", file->str);
+	GB_TRACE("filebrowser_on_drag_data_get - selection data is [%s]\n", file->str);
 	
 	/* Set the fully built path(s) as the selection data */
 	gtk_selection_data_set(selection_data, selection_data->target, 8, 
@@ -693,7 +694,7 @@ filebrowser_setup_tree(GtkTreeView* dirtree, GtkTreeView* filelist)
 	    
 	/* Enable the file list as a drag source */	
     gtk_drag_source_set(GTK_WIDGET(dirtree), GDK_BUTTON1_MASK, targetentries,
-		3, GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK);
+		TARGET_COUNT, GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK);
 
 	g_signal_connect(dirtree, "drag_data_get", 
 		G_CALLBACK(filebrowser_on_drag_data_get), store);
@@ -803,7 +804,7 @@ filebrowser_setup_list(GtkTreeView* filelist)
 	/* 3rd column to add the human readeable size to */
 	col = gtk_tree_view_column_new();
     gtk_tree_view_column_set_resizable(col, TRUE);
-	gtk_tree_view_column_set_title(col, "Size");
+	gtk_tree_view_column_set_title(col, _("Size"));
 	renderer = gtk_cell_renderer_text_new();
 	gtk_tree_view_column_pack_start(col, renderer, TRUE);
 	gtk_tree_view_column_set_attributes(col, renderer, "text", FL_COL_HUMANSIZE,NULL);
@@ -833,9 +834,9 @@ filebrowser_setup_list(GtkTreeView* filelist)
     gtk_tree_selection_set_mode(gtk_tree_view_get_selection(filelist),
 		GTK_SELECTION_MULTIPLE);
 
-	/* Enable the file list as a drag source */	
+	/* Enable the file list as a drag source */
     gtk_drag_source_set(GTK_WIDGET(filelist), GDK_BUTTON1_MASK, targetentries,
-		3, GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK);
+        TARGET_COUNT, GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK);
 
 	/* Connect the function to handle the drag data */
     g_signal_connect(filelist, "drag_data_get",
@@ -888,6 +889,11 @@ void
 filebrowser_new()
 {
 	GB_LOG_FUNC
+    
+    ROOT_LABEL = _(ROOT_LABEL);
+    HOME_LABEL = _(HOME_LABEL);
+    EMPTY_LABEL = _(EMPTY_LABEL);
+    DIRECTORY = _(DIRECTORY);
 	
 	GtkTreeView* dirtree = 
 		GTK_TREE_VIEW(glade_xml_get_widget(gnomebaker_getxml(), widget_browser_dirtree));
