@@ -41,8 +41,11 @@ static const gchar* const widget_gnomebaker = "GnomeBaker";
 static const gchar* const widget_project_notebook = "notebook1";
 static const gchar* const widget_appbar = "appbar1";
 static const gchar* const widget_import = "toolbutton5";
+static const gchar* const widget_menu_import = "import_session1";
 static const gchar* const widget_up = "toolbutton8";
+static const gchar* const widget_menu_up = "move_selection_up1";
 static const gchar* const widget_down = "toolbutton9";
+static const gchar* const widget_menu_down = "move_selection_down1";
 static const gchar* const widget_top_toolbar_dock = "bonobodockitem4";
 static const gchar* const widget_top_toolbar = "toolbar3";
 static const gchar* const widget_middle_toolbar = "toolbar4";
@@ -142,14 +145,15 @@ gnomebaker_new()
 	
 	GtkWidget *notebook = glade_xml_get_widget(xml, widget_project_notebook);
 	gtk_notebook_remove_page(GTK_NOTEBOOK(notebook), -1);
+    
+    gtk_widget_show_all (main_window);
 	
+    /* Check preferences to see if we'll show/hide the file browser */
 	GtkWidget* checkmenuitem = glade_xml_get_widget(xml, widget_show_browser_menu);
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(checkmenuitem),
 		preferences_get_bool(GB_SHOW_FILE_BROWSER));
 	g_signal_emit_by_name(checkmenuitem, "toggled", checkmenuitem, NULL);	
 
-	gtk_widget_show_all (main_window);
-
 	return main_window;
 }
 
@@ -163,7 +167,7 @@ gnomebaker_delete(GtkWidget* self)
     gb_common_finalise();
 	gtk_widget_hide(self);
 	gtk_widget_destroy(self);
-	g_free(xml);
+	g_object_unref(xml);
 }
 
 
@@ -172,7 +176,7 @@ gnomebaker_show_msg_dlg(GtkWindow* parent, GtkMessageType type,
     GtkButtonsType buttons, GtkButtonsType additional, const gchar * message)
 {
 	GB_LOG_FUNC
-	GB_TRACE("MessageDialog message [%s]", message);		
+	GB_TRACE("gnomebaker_show_msg_dlg - message [%s]\n", message);		
 	
 	GtkWidget *dialog = gtk_message_dialog_new(
 		parent == NULL ? GTK_WINDOW(glade_xml_get_widget(xml, widget_gnomebaker)) : parent, 
@@ -275,19 +279,11 @@ gnomebaker_on_burn_iso(gpointer widget, gpointer user_data)
     gtk_file_filter_add_custom(imagefilter, GTK_FILE_FILTER_FILENAME | GTK_FILE_FILTER_MIME_TYPE,
         gnomebaker_cd_image_file_filter, NULL, NULL);
     gtk_file_filter_set_name(imagefilter,_("CD Image files"));
-	const gchar* file = gbcommon_show_file_chooser(_("Please select a CD image file."), imagefilter);
+	gchar* file = gbcommon_show_file_chooser(_("Please select a CD image file."), 
+        GTK_FILE_CHOOSER_ACTION_OPEN, imagefilter, TRUE, NULL);
 	if(file != NULL)
 		burn_cd_image_file(file);
-}
-
-
-static gboolean
-gnomebaker_dvd_image_file_filter(const GtkFileFilterInfo *filter_info, gpointer data)
-{
-    GB_LOG_FUNC
-    if(g_ascii_strcasecmp(filter_info->mime_type, "application/x-cd-image") == 0)
-        return TRUE;
-    return FALSE;
+    g_free(file);
 }
 
 
@@ -297,11 +293,13 @@ gnomebaker_on_burn_dvd_iso(gpointer widget, gpointer user_data)
 	GB_LOG_FUNC
     GtkFileFilter *imagefilter = gtk_file_filter_new();
     gtk_file_filter_add_custom(imagefilter, GTK_FILE_FILTER_MIME_TYPE,
-        gnomebaker_dvd_image_file_filter, NULL, NULL);
+        gbcommon_iso_file_filter, NULL, NULL);
     gtk_file_filter_set_name(imagefilter,_("DVD Image files"));
-	const gchar* file = gbcommon_show_file_chooser(_("Please select a DVD image file."), imagefilter);
+	gchar* file = gbcommon_show_file_chooser(_("Please select a DVD image file."), 
+        GTK_FILE_CHOOSER_ACTION_OPEN, imagefilter, TRUE, NULL);
 	if(file != NULL)
         burn_dvd_iso(file);
+    g_free(file);
 }
 
 
@@ -368,9 +366,9 @@ gnomebaker_on_about(GtkMenuItem * menuitem, gpointer user_data)
 	{
 		const gchar* authors[] = {"Luke Biddell", "Christoffer Sørensen", "Razvan Gavril", "Isak Savo", "Adam Biddell (Sounds)", NULL};
 		const gchar* documenters[] = {"Milen Dzhumerov", NULL};
-		about = gnome_about_new(_("GnomeBaker"), VERSION, "GPL", 
+		about = gnome_about_new(_("GnomeBaker"), PACKAGE_VERSION, "GPL", 
 			_("Simple CD/DVD Burning for Gnome"), authors, documenters, _("translator_credits"), 
-			gdk_pixbuf_new_from_file(PACKAGE_PIXMAPS_DIR"/splash_2.png", NULL));
+			gdk_pixbuf_new_from_file(IMAGEDIR"/splash_2.png", NULL));
 		g_object_add_weak_pointer(G_OBJECT(about), (void**)&about);
 		gtk_widget_show(about);	
 	}
@@ -450,7 +448,7 @@ gnomebaker_on_add_files(gpointer widget, gpointer user_data)
             for(; file != NULL; file = file->next)
             {
                 /* hand ownership of the data to the gchar** which we free later */
-                GB_TRACE("index [%d] file [%s]", index, (gchar*)file->data);
+                GB_TRACE("gnomebaker_on_add_files - index [%d] file [%s]\n", index, (gchar*)file->data);
                 g_string_append(text, (gchar*)file->data);
                 g_string_append(text, "\n");
                 g_free((gchar*)file->data);
@@ -461,7 +459,7 @@ gnomebaker_on_add_files(gpointer widget, gpointer user_data)
             selection_data = g_new0(GtkSelectionData, 1);	                 
             gtk_selection_data_set(selection_data, selection_data->target, 8, 
                 (const guchar*)text->str, strlen(text->str) * sizeof(gchar));
-            GB_TRACE("[%s]", selection_data->data);
+            GB_TRACE("gnomebaker_on_add_files - [%s]\n", selection_data->data);
             g_slist_free(files);
             /*g_strfreev(uris);*/
             g_string_free(text, TRUE);
@@ -596,7 +594,7 @@ gnomebaker_on_help(gpointer widget, gpointer user_data)
 	GB_LOG_FUNC
 	GError *error = NULL;
 	gnome_help_display("gnomebaker", NULL, &error);
-	if(error)
+	if(error != NULL)
 	{
 		gnomebaker_show_msg_dlg(NULL, GTK_MESSAGE_ERROR,GTK_BUTTONS_CLOSE,
 			GTK_BUTTONS_NONE, error->message);
@@ -613,19 +611,28 @@ gnomebaker_on_notebook_switch_page(GtkNotebook *notebook,
 {
     GB_LOG_FUNC
     GtkWidget* import = glade_xml_get_widget(xml, widget_import);
+    GtkWidget* menu_import = glade_xml_get_widget(xml, widget_menu_import);
     GtkWidget* up = glade_xml_get_widget(xml, widget_up);
     GtkWidget* down = glade_xml_get_widget(xml, widget_down);
+    GtkWidget* menu_up = glade_xml_get_widget(xml, widget_menu_up);
+    GtkWidget* menu_down = glade_xml_get_widget(xml, widget_menu_down);
     switch(page_num) 
     {
         case 0:
             gtk_widget_set_sensitive(import, TRUE);
+            gtk_widget_set_sensitive(menu_import, TRUE);
             gtk_widget_set_sensitive(up, FALSE);
             gtk_widget_set_sensitive(down, FALSE);
+            gtk_widget_set_sensitive(menu_up, FALSE);
+            gtk_widget_set_sensitive(menu_down, FALSE);
             break;
         case 1:
             gtk_widget_set_sensitive(import, FALSE);
+            gtk_widget_set_sensitive(menu_import, FALSE);
             gtk_widget_set_sensitive(up, TRUE);
             gtk_widget_set_sensitive(down, TRUE);
+            gtk_widget_set_sensitive(menu_up, TRUE);
+            gtk_widget_set_sensitive(menu_down, TRUE);
             break;
         default:{}
     };
@@ -695,7 +702,8 @@ gnomebaker_on_import_playlist(gpointer widget, gpointer user_data)
     gtk_file_filter_add_custom(imagefilter, GTK_FILE_FILTER_FILENAME | GTK_FILE_FILTER_MIME_TYPE,
         gnomebaker_playlist_file_filter, NULL, NULL);
     gtk_file_filter_set_name(imagefilter,_("Playlist files"));
-    const gchar* file = gbcommon_show_file_chooser(_("Please select a playlist."), imagefilter);
+    gchar* file = gbcommon_show_file_chooser(_("Please select a playlist."), 
+            GTK_FILE_CHOOSER_ACTION_OPEN, imagefilter, TRUE, NULL);
     if(file != NULL)
     {
         if(audiocd_import_playlist(file))
@@ -704,6 +712,72 @@ gnomebaker_on_import_playlist(gpointer widget, gpointer user_data)
             gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), 1);
         }
     }
+    g_free(file);
 }
 
 
+void /* libglade callback */
+gnomebaker_on_export_playlist(gpointer widget, gpointer user_data)
+{
+    GB_LOG_FUNC
+
+    GtkFileFilter *imagefilter = gtk_file_filter_new();
+    gtk_file_filter_add_custom(imagefilter, GTK_FILE_FILTER_FILENAME | GTK_FILE_FILTER_MIME_TYPE,
+            gnomebaker_playlist_file_filter, NULL, NULL);
+    gtk_file_filter_set_name(imagefilter,_("Playlist files"));
+    
+    GtkWidget *filetypes_combo = gtk_combo_box_new_text();
+    gtk_combo_box_append_text(GTK_COMBO_BOX(filetypes_combo), ".m3u");
+    gtk_combo_box_append_text(GTK_COMBO_BOX(filetypes_combo), ".pls");
+    gtk_combo_box_set_active(GTK_COMBO_BOX(filetypes_combo), 0);
+            
+    gchar* file = gbcommon_show_file_chooser(_("Save playlist as..."),
+            GTK_FILE_CHOOSER_ACTION_SAVE, imagefilter, FALSE, GTK_COMBO_BOX(filetypes_combo));
+    audiocd_export_playlist(file);
+    g_free(file);
+}
+
+
+static gboolean
+gnomebaker_project_file_filter(const GtkFileFilterInfo *filter_info, gpointer data)
+{
+    GB_LOG_FUNC
+    return gbcommon_str_has_suffix(filter_info->filename, ".gbp");
+}
+
+
+void /* libglade callback */
+gnomebaker_on_open_project(gpointer widget, gpointer user_data)
+{
+    GB_LOG_FUNC   
+    
+    GtkFileFilter *imagefilter = gtk_file_filter_new();
+    gtk_file_filter_add_custom(imagefilter, GTK_FILE_FILTER_FILENAME,
+        gnomebaker_playlist_file_filter, NULL, NULL);
+    gtk_file_filter_set_name(imagefilter,_("Project files"));
+    gchar* file = gbcommon_show_file_chooser(_("Please select a project file."), GTK_FILE_CHOOSER_ACTION_OPEN, imagefilter, FALSE, NULL);
+    if(file != NULL);
+    g_free(file);
+}
+
+
+void /* libglade callback */
+gnomebaker_on_save_project(gpointer widget, gpointer user_data)
+{
+    GB_LOG_FUNC   
+}
+
+
+void /* libglade callback */
+gnomebaker_on_save_project_as(gpointer widget, gpointer user_data)
+{
+    GB_LOG_FUNC   
+}
+
+
+void /* libglade callback */
+gnomebaker_on_save_all(gpointer widget, gpointer user_data)
+{
+    GB_LOG_FUNC   
+}
+
